// ***************************************************************************
// FileIO Class
// 
//
// Class Description:  
// ------------------
// Purpose:  This class is a utility class for file input and output for the Atom 
// system.  It perfoms both read and write on sequential files.  Finally, it has
// a method to clean up temporary files that it creates.
//
// System Relationship:  Intercommunication with the whole system is maximal, not
// by FileIO but by other classes (Atom, SclInterface, ToolInterface, AddWindow)
// using the utility methods in FileIO.  The majority of the relationships deals 
// with typical file activities, addition, deletion, appending, creation, and removal.
//
// Requirements:  This class requires the Java Developemnt Kit (JDK) 1.0.
// Aside from this there are no special algorithms, or constraints.  
//
// Pre-Conditions:  True.  Some methods require the input file name.
//
// Post Conditions: True.
//
// Error Handling:  Addressed with the try and catch clause.
//
//
// Class Specification:
// --------------------
// This class must open a file read from it and close the file.  Moreover, information 
// retrieved from a file is then appended to another file and that file is copied to 
// the original source file.  File removal is another important feature of this class.  
// It is capable of checking for a file's existence, as well.  The following is a list 
// of requirements used by this class:
//
//    *  Open a file for reading, writing, or both
//    *  Close a file
//    *  Remove a file
//    *  Retrieve data from a file
//
// This class does not have a constructor.
//
//
// Methods
// -------
//      public void appendText(String file, String text, String description)
//      public void removeText(String ifile, String ofile, String removeText) 
//      public void copyFile(String src, String dest) 
//      public String getText(String src, String name)
//      public void replaceText(String ifile, String ofile, String oldName, String newName,
//      			String description)
//      public void AtplAppendText(String file, String keyword, String data) 
//      public void AtplRemoveRecord(String ifile, String ofile, String removeText)
//      public void AtplGetRecord(String src, String name, TextArea txtArea)
//      public void AtplReplaceRecord(String ifile, String ofile, String record, String keyword,
//		         	      String newInfo)
//      public String AtplGetField(String src, String record, String keyword)
//      public void appendSclAssignment(String auditTool, String sclType, String info, 
//				        String directory)
//      public void removeSclAssignment(String auditTool, String sclType) 
//      public void startScl(String sclType)
//      public void stopScl(String sclType)
//      public void echoText(String src)
//      public void echoText(String src, TextArea txtArea)
//      public void startTool(String dir, String tool, String parmInfo) 
//      public void stopTool(String tool)
//      public void cleanUp() 
//
// *****************************************************************************

import java.io.*;
import java.lang.*;
import java.util.*;
import java.awt.*;

public class FileIO {

  // *********************************************************************************
  // public void appendText(String file, String text, String description)
  //
  // Method Description:  This method appends text to the end of a file.
  //
  // Method Specification:  This method requires a source file, and text to append to
  // the end of a file.  The text refers to the Scl Name and the description refers to
  // the Scl Description.  The two fields are delimited by the "^"
  //   
  // This method will be accessed several times.  Every time a Scl is added, or configured.
  //
  // Pre-Condition:  The source file must exist.  Both text and description should be 
  // valid alpha-numeric characters, if this fails the method still works.
  //
  // Post-Condition: The text is appended to the end of the source file.
  // 
  // *********************************************************************************

  public void appendText(String file, String text, String description) {
    try {
      RandomAccessFile ofile = new RandomAccessFile(file, "rw");

      ofile.seek(ofile.length());
      ofile.writeBytes(text);
      ofile.writeByte('^');
      ofile.writeBytes(description);
      ofile.writeByte('\n');

      ofile.close();
    }
    catch (IOException e) {
      System.err.println("Error occurred in FileIO.appendText(String, String)");
      return;
    }

  } // appendText


  // *********************************************************************************
  // public void removeText(String ifile, String ofile, String removeText
  //
  // Method Description:  This method will remove text from a source file.
  //
  // Method Specification:  This method will remove text from a source file, it uses the 
  // ofile as a temp file.  The removeText string is the text to be removed from the file.
  // This file calls the copyFile() method.
  //   
  // This method will be accessed several times.  Every time a Scl is added, or configured,
  // or removed.
  //
  // Pre-Condition:  The source file must exist.  If the removeText is not a valid match
  // then nothing will happen and system execution will continue as normal. 
  //
  // Post-Condition: The text is removed from the source file.
  // 
  // *********************************************************************************

  public void removeText(String ifile, String ofile, String removeText) {
    String token;
    String line;

    try {

      // BufferedReader input = new BufferedReader(new InputStreamReader(process.getInputStream()));

      BufferedReader infile   = new BufferedReader(new FileReader(ifile));
      DataOutputStream outfile = new DataOutputStream(new FileOutputStream(ofile));
      
      // While the file has text and is not eof parse each line into tokens 
      // delimited by the "^"
      while ((line = infile.readLine()) != null) {
	StringTokenizer inputString = new StringTokenizer(line, "^");
	
	token = inputString.nextToken();

	if (!(token.equals(removeText))) {
	  outfile.writeBytes(line);
	  outfile.writeByte('\n');
	}
      } // while
      
      infile.close();
      outfile.close(); 
      
      copyFile(ofile, ifile);

    } // try

    catch (IOException e) {
      System.err.println("Error occurred in FileIO.removeText(String, String)");
      return;
    }
    
  } // removeText


  // *********************************************************************************
  // public void copyFile(String src, String dest)
  //
  // Method Description:  This method will copy a source file to a destination file.
  //
  // Method Specification:  This method takes a source file and copies it to a destination
  // file.  It does this by reading in each line of the source file and writing each line
  // out to a destination file.
  //   
  // This method will be accessed several times.  
  //
  // Pre-Condition:  The source file must exist.  The destination file does not need to
  // exist.
  //
  // Post-Condition: The source file is copied to the destination file.
  // 
  // *********************************************************************************

  public void copyFile(String src, String dest) {
    String line;

    try {
      BufferedReader srcFile  = new BufferedReader(new FileReader(src));
      DataOutputStream destFile = new DataOutputStream(new FileOutputStream(dest));    

      while ((line = srcFile.readLine()) != null) {
	destFile.writeBytes(line);
	destFile.writeByte('\n');
      } // while
      
      srcFile.close();
      destFile.close();      
    } // try
    
    catch (IOException e) {
      System.err.println("Error occurred in FileIO.copyFile(String, String)");
      return;
    }
  }  // copyFile


  // *********************************************************************************
  // public String getText(String src, String name)
  //
  // Method Description:  This method will get text from the a file based on a string 
  // name.  Moreover, this file retreives the description of a scl name from the 
  // scl_types.dat file.
  //
  // Method Specification:  This method retrieves text from a source file, which is the
  // scl_types.dat file.  The information returned by this function is the description of 
  // a Scl type.
  //   
  // This method will be accessed frequently. 
  //
  // Pre-Condition:  The source file must exist.  If the name is not a valid match
  // then nothing will happen and system execution will continue as normal. 
  //
  // Post-Condition: The description information is returned.
  // 
  // *********************************************************************************
 
  public String getText(String src, String name) {
    String line;
    String token;
    String field = "";

    try {
      BufferedReader srcFile  = new BufferedReader(new FileReader(src));  

      while ((line = srcFile.readLine()) != null) {
	StringTokenizer inputString = new StringTokenizer(line, "^");

	token = inputString.nextToken();
	
	if (token.equals(name)) {
	  field = inputString.nextToken();
	  break;
	}
      } // while
      
      srcFile.close();
    } // try
    
    catch (IOException e) {
      System.err.println("Error occurred in FileIO.getText(String, String)");
      return field;
    }
    
    return field;

  } // getText


  // *********************************************************************************
  // public void replaceText(String ifile, String ofile, String oldName, String newName,
  //			     String description)
  //
  // Method Description:  This method will replace text in a source file.  Moreover, it
  // will replace text information in the scl_types.dat file.
  //
  // Method Specification:  This method replaces text in a source file.  It implements the
  // removeText(), copyFile(), and appendText() methods previously defined. 
  //   
  // This method will be frequently.  
  //
  // Pre-Condition:  The source file must exist.  
  //
  // Post-Condition: The text is replaced in the source file.
  // 
  // *********************************************************************************

  public void replaceText(String ifile, String ofile, String oldName, String newName,
			  String description) {

    removeText(ifile, ofile, oldName);
    copyFile(ofile, ifile);
    appendText(ifile, newName, description);

  } // replaceText


  // *********************************************************************************
  // public void AtplAppendText(String file, String keyword, String data)
  //
  // Method Description:  This method appends text to the end of a file.  It implements
  // the audit tool parsing language (Aptl) construct.
  //
  // Method Specification:  This method requires a source file, a Atpl keyword, and text
  // to append to the end of a file.  The keyword is currently defined in the ToolInterface
  // class.  The basic format of a Atpl file is <keyword> <text>.  The end of a record is
  // defined by the ENDSYMBOL.
  //   
  // This method will be accessed several times.  
  //
  // Pre-Condition:  The source file must exist and the Atpl keywords must be declared and 
  // defined. 
  //
  // Post-Condition: The text is appended to the source file.
  // 
  // *********************************************************************************

  public void AtplAppendText(String file, String keyword, String data) {
    try {
      RandomAccessFile ofile = new RandomAccessFile(file, "rw");

      if (keyword.equals("END")) {
	ofile.seek(ofile.length());
	ofile.writeBytes(data);
	ofile.writeByte('\n');
      }
      else {
	ofile.seek(ofile.length());
	ofile.writeBytes(keyword + " " + data);
	ofile.writeByte('\n');
      }
      
     ofile.close();
    }
    catch (IOException e) {
      System.err.println("Error occurred in FileIO.ATPLappendText()");
      return;
    }

  } // ATPL appendText


  // *********************************************************************************
  // public void AtplRemoveRecord(String ifile, String ofile, String removeText) 
  //
  // Method Description:  This method remove a record from a source file using the Atpl 
  // format.
  //
  // Method Specification:  Using the Atpl format this method will remove a whole record 
  // from the source file.  The whole record starts with NAME and ends with the ENDSYMBOL.
  // This method calls the copyFile() method.
  //
  // This method will be accessed several times.  
  //
  // Pre-Condition:  The source file must exist. The source file must be in the proper Atpl
  // format.
  //
  // Post-Condition: The record is removed from the source file.
  // 
  // ********************************************************************************* 

  public void AtplRemoveRecord(String ifile, String ofile, String removeText) {
    String token;
    String line;

    try {
      BufferedReader infile   = new BufferedReader(new FileReader(ifile));
      DataOutputStream outfile = new DataOutputStream(new FileOutputStream(ofile));
      
      // 
      // While the file is not EOF
      //    * Get a line of the source file
      //    * Parse the input line using whitespace as a token delimiter
      //
      while ((line = infile.readLine()) != null) {
	StringTokenizer inputString = new StringTokenizer(line, " ");
	String  completeToken = "";
       
	token = inputString.nextToken();

	if (token.equals(ToolInterface.NAME)) {                 // Keyword NAME = token NAME
	 
	  while (inputString.hasMoreTokens()) {                 // Retrieve Full token 
	    token = inputString.nextToken();                    // Complete Token is the full name
	    completeToken = completeToken+" "+token;
	  } // while
	  completeToken = completeToken.trim();
	  //
	  // If the Complete Token is matches the removeText passed into this 
	  // method, then remove this record or do not write out to the temp file.
	  //

	  if (completeToken.equals(removeText)) {
	    while(!(line = infile.readLine()).equals(ToolInterface.ENDSYMBOL)) {}
	    line = infile.readLine();	    	   
	  }
	} // if
	
	// Write valid information out to the temp file
	if(!(line == null)) {
	  outfile.writeBytes(line);
	  outfile.writeByte('\n');
	}
      } // while
      
      infile.close();
      outfile.close(); 
      
      // Copy temp file back to source file
      copyFile(ofile, ifile);

    } // try
    
    catch (IOException e) {
      System.err.println("Error occurred in FileIO.ATPLRemoveRecord()");
      return;
    }

  } // ATPL Remove Record 


  // *********************************************************************************
  // public void AtplGetRecord(String src, String name, TextArea txtArea)
  //
  // Method Description:  This will display an Atpl record in the TextArea passed to 
  // this method.
  //
  // Method Specification:  This method searches through the source file for the record
  // Name that matches the name passed to it.  If it is a match then the record's 
  // information is displayed in the TextArea passed to this method.
  //   
  // This method will be accessed several times.  
  //
  // Pre-Condition:  The source file must exist.  
  //
  // Post-Condition: The record information is displayed in the TextArea.
  // 
  // *********************************************************************************
 
  public void AtplGetRecord(String src, String name, TextArea txtArea) {
    String line;
    String token;
    String field = "";

    try {
      BufferedReader srcFile  = new BufferedReader(new FileReader(src));  

      // 
      // While the file is not EOF
      //    * Get a line of the source file
      //    * Parse the input line using whitespace as a token delimiter
      //
      while ((line = srcFile.readLine()) != null) {
	StringTokenizer inputString = new StringTokenizer(line, " ");
	String completeToken = "";
	token = inputString.nextToken();

	if (token.equals(ToolInterface.NAME)) {                      // Keyword NAME = token NAME
	  while (inputString.hasMoreTokens()) {                      // Retrieve Full token
	    token = inputString.nextToken();                         // Complete Token is the full name
	    completeToken = completeToken+" "+token;
	  } // while
	  completeToken = completeToken.trim();

	  //
	  // If the Complete Token is matches the name passed into this 
	  // method, then display the record information in the TextArea
	  //
	  if (completeToken.equals(name)) {   
	    while(!(line = srcFile.readLine()).equals(ToolInterface.ENDSYMBOL)) {
	      txtArea.append(line + "\n");	      
	    } 
	  }
	} // if 
      } // while
  
      srcFile.close();
    } // try
    
    catch (IOException e) {
      System.err.println("Error occurred in FileIO.AtplGetRecord(String, String)");
      return;
    }

  } // Atpl Get Record

  
  // *********************************************************************************
  // public void AtplReplaceRecord(String ifile, String ofile, String record, String keyword,
  //				   String newInfo)
  //
  // Method Description:  This method will replace a field in a Atpl record.
  //
  // Method Specification:  This method will search through the Atpl source file for the 
  // specified record and replace the appropriate field based on the keyword.  The current
  // information is replaced with the newInfo.  This method implements the copyFile() method.
  //   
  // This method will be accessed several times.  
  //
  // Pre-Condition:  The source file must exist. 
  //
  // Post-Condition: The record is modified to reflect the new configuration.
  // 
  // *********************************************************************************

  public void AtplReplaceRecord(String ifile, String ofile, String record, String keyword,
				String newInfo) {
    String token;
    String keyToken;
    String line;

    try {
      BufferedReader infile  = new BufferedReader(new FileReader(ifile));
      DataOutputStream outfile = new DataOutputStream(new FileOutputStream(ofile));

      // 
      // While the file is not EOF
      //    * Get a line of the source file
      //    * Parse the input line using whitespace as a token delimiter
      //      
      while ((line = infile.readLine()) != null) {
	StringTokenizer inputString = new StringTokenizer(line, " ");
	String  completeToken = "";

	token = inputString.nextToken();          // Normal  token
	keyToken = token;                         // Keyword token

	if (token.equals(ToolInterface.NAME)) {              // Keyword NAME = token NAME
	  while (inputString.hasMoreTokens()) {              // Retrieve Full token
	    token = inputString.nextToken();                 // Complete Token is the full name
	    completeToken = completeToken+" "+token;
	  } // while

	  //
	  // If the completeToken, which is the record name, matches the record 
	  // we are looking for then search for the keyword field to replace
	  // and write out the newInfo to the output file.
	  //
	  if (completeToken.equals(record)) {                
	    do {
	      if (keyToken.equals(keyword)) {                   
		outfile.writeBytes(keyword +"  "+newInfo);    // This is the field we're looking for
	      }
	      else {
		outfile.writeBytes(line);                     // Not the field we're looking for
	      }
	      outfile.writeByte('\n');
	    
	      if ((line = infile.readLine()) != null) {
	        inputString = new StringTokenizer(line, " ");
		keyToken = inputString.nextToken();
	      }
	    } while (!(keyToken.equals(ToolInterface.ENDSYMBOL)));
	    
	  } // if
	} // if Beginning of record

	outfile.writeBytes(line);
	outfile.writeByte('\n');   	
	
      } // while 
      
      outfile.close();
      infile.close();

      copyFile(ofile, ifile);       
    } // try
	
    catch (IOException e) {
      System.err.println("Error occurred in FileIO.AtplReplaceRecord()");
      return;
    }

  } // AtplReplaceRecord 


  // *********************************************************************************
  // public String AtplGetField(String src, String record, String keyword)
  //
  // Method Description:  This method will get a field of a Atpl record from a source
  // file.
  //
  // Method Specification:   This method will search through the Atpl source file for the 
  // specified record and extract the appropriate field based on the keyword.  The extracted
  // field will be returned by this method.
  //   
  // This method will be accessed several times.  
  //
  // Pre-Condition:  The source file must exist.  
  //
  // Post-Condition: The text is returned by this method.
  // 
  // *********************************************************************************
 
  public String AtplGetField(String src, String record, String keyword) {
    String token;
    String keyToken;
    String line;
    String returnString = "";

    try {
      BufferedReader infile  = new BufferedReader(new FileReader(src));

      // 
      // While the file is not EOF
      //    * Get a line of the source file
      //    * Parse the input line using whitespace as a token delimiter
      //      
      while ((line = infile.readLine()) != null) {
	StringTokenizer inputString = new StringTokenizer(line, " ");
	String  completeToken = "";

	token = inputString.nextToken();                  // Normal  token
	keyToken = token;                                 // Keyword token

	if (token.equals(ToolInterface.NAME)) {           // Keyword NAME = token NAME
	  while (inputString.hasMoreTokens()) {           // Retrieve Full token
	    token = inputString.nextToken();              // Complete Token is the full name
	    completeToken = completeToken+" "+token;
	  } // while
	  completeToken = completeToken.trim();

	  //
	  // If the completeToken, which is the record name, matches the record 
	  // we are looking for then search for the keyword field to return.
	  //
	  if (completeToken.equals(record)) {                   	  
	    do {
	      if (keyToken.equals(keyword)) {              // This is the field we're looking for
		while (inputString.hasMoreTokens()) {      // Get full field, all the tokens
		  token = inputString.nextToken();
		  returnString = returnString+" "+token;
		} // while
	      }
     
	      if ((line = infile.readLine()) != null) {
		inputString = new StringTokenizer(line, " ");
		keyToken = inputString.nextToken();
	      }
	    } while (!(keyToken.equals(ToolInterface.ENDSYMBOL)));
	    
	  } // if
	} // if Beginning of record
      } // while 
      
      infile.close();
    } // try
	
    catch (IOException e) {
      System.err.println("Error occurred in FileIO.AtplGetField()");
      return returnString;
    }

    // Trim the whitespace off the ends of the string
    returnString = returnString.trim();   
    return returnString;

  } // Atpl Get Field 


  // *********************************************************************************
  // public void appendSclAssignment(String auditTool, String sclType, String info, 
  //				     String directory)
  //
  // Method Description:  This method append text to a file.  Moreover, it will append
  // text to a Scl_Name file, according to the audit tool assignment.
  //
  // Method Specification:  When an audit tool is assigned to a Scl it needs to be written
  // out to that Scl file.  This file will create that file if it does not exist, and 
  // append the audit tool information to the file.
  //
  // This method will be accessed several times, dependent on the number of Scl assingments.  
  //
  // Pre-Condition:  True.
  //
  // Post-Condition: The information is assigned to proper Scl_Name file.
  //
  // *********************************************************************************

  public void appendSclAssignment(String auditTool, String sclType, String info, 
				  String directory) {

    // File name declaration assignment
    String file = Atom.sclLevelDir+"scl_"+sclType;

    // Validation  (text)    
    file = file.replace(' ','_');                // replace spaces with underscores
    info = info.trim();                          // trim off any whitespace - on either end
    directory = directory.trim();                // trim off any whitespace - on either end
    if (!(directory.endsWith("/"))) {            // if directory does not end with "/" then put it there
      directory = directory+"/";
    }

    try {
      RandomAccessFile ofile = new RandomAccessFile(file, "rw");
      
      ofile.seek(ofile.length());
      ofile.writeBytes(directory);
      ofile.writeByte('^');
      ofile.writeBytes(info);
      ofile.writeByte('\n');
      ofile.close();
    }
    catch (IOException e) {
      System.err.println("Error occurred in FileIO.appendSclAssignment()");
      return;
    } 

  } // appendSclAssignment
  
  
  // *********************************************************************************
  // public void removeSclAssignment(String auditTool, String sclType)
  //
  // Method Description:  This method removes a certain record from a file.  Moreover,
  // it will remove a record (one single line) from the scl_Name file.
  //
  // Method Specification:  When an audit tool is un-assigned it will be removed from its
  // corresponding scl_Name file.  This method removes the assignment from the scl_Name 
  // file.
  //
  // This method may be accessed frequently.  
  //
  // Pre-Condition:  The source file must exist.  If the remove text (sclType) is not a 
  // valid match then nothing will happen and system execution will continue as normal. 
  //
  // Post-Condition: The text is removed from the source file.
  // 
  // *********************************************************************************
 
  public void removeSclAssignment(String auditTool, String sclType) {    
    
    // File assignments
    String file     = Atom.sclLevelDir+"scl_"+sclType;
    String tempFile = Atom.sclLevelTemp;

    String line;
    String token;

    // Validation  (text)
    file = file.replace(' ','_');                     // replace whitespace with underscores
    auditTool = auditTool.trim();                     // trim off any whitespace
    
    // Check for file existence
    File checkFile = new File(file);
    if (!(checkFile.exists())) {
      Atom.systemStatusTextArea.append("This file type does not exist. \n");
      return;
    }

    try {
      BufferedReader ifile  = new BufferedReader(new FileReader(file));
      DataOutputStream ofile = new DataOutputStream(new FileOutputStream(tempFile));

      // 
      // While the file is not EOF
      //    * Get a line of the source file
      //    * Parse the input line using the carrot as a token delimiter
      //            
      while ((line = ifile.readLine()) != null) {
	StringTokenizer inputString = new StringTokenizer(line, "^");
	
	token = inputString.nextToken();                // directory
	token = inputString.nextToken();                // audit tool

   	if (!(token.startsWith(auditTool))) {	  
	  ofile.writeBytes(line);
	  ofile.writeByte('\n');
	}
     }
      ifile.close();
      ofile.close();

      copyFile(tempFile, file);
    }
    catch (IOException e) {      
      String err = e.toString();   
      Atom.systemStatusTextArea.append("There is no assignment of this type.\n"); 
      System.err.println(err);      
      System.err.println("Error occurred in FileIO.removeSclAssignment()");
      return;
    } 

  }  // removeSclAssignment

  
  // *********************************************************************************
  // public void startScl(String sclType)
  //
  // Modified: 4/8/97
  //
  // *********************************************************************************
 
  public void startScl(String sclType) {

    // File location Assignment Scl Level DIR + Scl Type
    String sclfile     = Atom.sclLevelDir+"scl_"+sclType;

    // Validity check
    sclfile = sclfile.replace(' ', '_');

    // Garbage collection
    Runtime resource = Runtime.getRuntime();
    resource.gc();
      
    try {
      String pipeio;        // stdout collected from exec
      String cmdline;       // application to be executed

      File file = new File(sclfile);
      BufferedReader infile   = new BufferedReader(new FileReader(file));
      
      while ((cmdline = infile.readLine()) != null) {
	
	// Create Runtime and Process object and exec the process
	Process process = Runtime.getRuntime().exec(cmdline);
	
	// Notify System Status
	Atom.systemStatusTextArea.append("Exec: "+ cmdline +"\n");

	// Capture output from process
	BufferedReader input = new BufferedReader(new InputStreamReader(process.getInputStream()));
	
	while ((pipeio = input.readLine()) != null) {
	  System.out.println(pipeio);
	}

	input.close();
      }
    }
      catch (Exception err) {
	System.out.println("EXEC failed: " + err.toString());
	err.printStackTrace();
	return;
      }
  }  // startScl


  // *********************************************************************************
  // public void stopScl(String sclType)
  //
  // Method Description:  This method calls the stopTools script through the Runtime 
  // Object variable.  The stopTools script needs the file name of the scl to stop,
  // then the stopTools script will kill/stop each record (audit tool) in the scl_Name file.
  //
  // Method Specification:  This method calls the stopTools script located in the 
  // "./scripts/" directory.  stopTools requires the command line argument,
  // -file "directory+scl_Name", to stop the audit tools.  This script is a Perl script.
  // The -file command line argument is the name of the scl_Name file where all of the 
  // audit tools assigned to a scl level are stored.
  // 
  // This method will be accessed every time the user wants to stop a suite of audit tools.
  // This may be frequent or infrequent, depending on the network and network administrator.
  //
  // Pre-Condition:  The scl_Name file needs to be in the correct format.  The scl_Name file
  // should exist and information in the file should be correct - correct enough for the
  // system to execute it at the command line.
  //
  // Post-Condition: The audit tools in the scl_Name file are stopped.
  // 
  // ********************************************************************************* 

  public void stopScl(String sclType) {

    // File assignment
    String file     = Atom.sclLevelDir+"scl_"+sclType;
    String script   = Atom.stopScript;   
 
    // Validity check
    file = file.replace(' ', '_');

    String executionString = script+" -file "+file;
    
    try {

      //
      // Create Runtime and Process objects
      // Execute the process 
      // Wait for it to complete
      //
      Runtime tool = Runtime.getRuntime();
      Process process = tool.exec(executionString);
      process.waitFor();

      // Temporary information is that should be sent to the console is sent 
      // to its corresponding files.  These are temporary files.
      // STDOUT goes to stdout.log 
      // STDERR goes to stderr.log
      // Check stdout.log and stderr.log
      // Clean up - delete temporary files
      File stdout = new File(Atom.stdoutLog);
      if (stdout.exists()) {
	echoText(Atom.stdoutLog);
	stdout.delete();
      }
      File stderr = new File(Atom.stderrLog);
      if (stderr.exists()) {
	echoText(Atom.stderrLog, Atom.systemStatusTextArea);	
	stderr.delete();
      }
    }

    catch (Exception e) {
      String err = e.toString();
      System.err.println(err);
      System.err.println("Error occurred in FileIO.stopScl()");
      return;
    } 

  } // stopScl


  // *********************************************************************************
  // public void echoText(String src)
  //
  // Method Description:  This method simply echoes the text from a source file to
  // the systemStatusArea located in the Atom class, or main interface.
  //
  // Method Specification:  This method takes the source file and echoes to the 
  // systemStatusTextArea every line in the file.  When the Java system stuff works better
  // this method should become obsolete.
  //   
  // This method will be every time a security level is started or stopped. 
  //
  // Pre-Condition:  The source file must exist. 
  //
  // Post-Condition: The text is echoed to the systemStatusTextArea.
  // 
  // *********************************************************************************
 
  public void echoText(String src) {
    String line;
  
    try {
      BufferedReader srcFile  = new BufferedReader(new FileReader(src));  

      while ((line = srcFile.readLine()) != null) {
        System.out.println(line);
      } // while
      
      srcFile.close();
    } // try
    
    catch (IOException e) {
      System.err.println("Error occurred in FileIO.echoText(String)");
      return;
    }

  } // echoText



  // *********************************************************************************
  // public void echoText(String src, TextArea txtArea)
  //
  // Method Description:  This method simply echoes the text from a source file to
  // the systemStatusArea located in the Atom class, or main interface.
  //
  // Method Specification:  This method takes the source file and echoes to the 
  // systemStatusTextArea every line in the file.  When the Java system stuff works better
  // this method should become obsolete.
  //   
  // This method will be every time a security level is started or stopped. 
  //
  // Pre-Condition:  The source file must exist. 
  //
  // Post-Condition: The text is echoed to the systemStatusTextArea.
  // 
  // *********************************************************************************

  public void echoText(String src, TextArea txtArea) {
    String line;
  
    try {
      BufferedReader srcFile  = new BufferedReader(new FileReader(src));  

      while ((line = srcFile.readLine()) != null) {
	Atom.systemStatusTextArea.append(line+"\n");
      } // while
      
      srcFile.close();
    } // try
    
    catch (IOException e) {
      System.err.println("Error occurred in FileIO.echoText(String, TextArea)");
      return;
    }

  } // echoText

 
  // *********************************************************************************
  // public void startTool(String dir, String tool, String parmInfo)
  //
  // Method Description:  This method calls the startTools script through the Runtime 
  // Object variable.  The startTools script needs the directory name and the execution
  // command to start the audit tool, then the startTools script will execute the tool.
  //
  // Method Specification:  This method calls the startTools script located in the 
  // "./scripts/" directory.  startTools requires the command line arguments,
  // -dir "directory" and -cmd "command" to start the audit tools.  This script is a Perl script.
  // The -dir and -cmd will be combined to execute on the command line as an argument.
  //
  // This method will be accessed every time the user wants to start a single audit tool.
  // This may be frequent or infrequent, depending on the network user/administrator.
  //
  // Pre-Condition:  The dir and tool need to be valid for the system command to work.
  //
  // Post-Condition: The audit tool is started.
  // 
  // *********************************************************************************

  public void startTool(String dir, String tool, String parmInfo) {
  
    String script   = Atom.startScript;
    parmInfo = "\""+parmInfo+"\"";
    
    String executionString = script+" -dir "+dir+" -cmd "+parmInfo;
    System.out.println();

    try {

      //
      // Create Runtime and Process objects
      // Execute the process 
      // Wait for it to complete
      //
      Runtime auditTool = Runtime.getRuntime();
      Process process = auditTool.exec(executionString);
      process.waitFor();

      // Temporary information is that should be sent to the console is sent 
      // to its corresponding files.  These are temporary files.
      // STDOUT goes to stdout.log 
      // STDERR goes to stderr.log
      // Check stdout.log and stderr.log
      echoText(Atom.stdoutLog);
      echoText(Atom.stderrLog, Atom.systemStatusTextArea);

      // Clean up - delete temporary files
      File stdout = new File(Atom.stdoutLog);
      stdout.delete();
      File stderr = new File(Atom.stderrLog);
      stderr.delete();
    }

    catch (Exception e) {
      String err = e.toString();
      System.err.println(err);
      System.err.println("Error occurred in FileIO.startTool()");
      return;
    } 

  }  // startScl


  // *********************************************************************************
  // public void stopTool(String tool)
  //
  // Method Description:  This method calls the startTools script through the Runtime 
  // Object variable.  The startTools script needs the directory name and the execution
  // command to start the audit tool, then the startTools script will execute the tool.
  //
  // Method Specification:  This method calls the startTools script located in the 
  // "./scripts/" directory.  startTools requires the command line arguments,
  // -dir "directory" and -cmd "command" to start the audit tools.  This script is a Perl script.
  // The -dir and -cmd will be combined to execute on the command line as an argument.
  //
  // This method will be accessed every time the user wants to start a single audit tool.
  // This may be frequent or infrequent, depending on the network user/administrator.
  //
  // Pre-Condition:  The dir and tool need to be valid for the system command to work.
  //
  // Post-Condition: The audit tool is started.
  // 
  // *********************************************************************************

  public void stopTool(String tool) {
  
    String script   = Atom.stopScript;    
    String executionString = script+" -cmd "+tool;
    
    try {

      //
      // Create Runtime and Process objects
      // Execute the process 
      // Wait for it to complete
      //
      Runtime auditTool = Runtime.getRuntime();
      Process process = auditTool.exec(executionString);
      process.waitFor();

      // Temporary information is that should be sent to the console is sent 
      // to its corresponding files.  These are temporary files.
      // STDOUT goes to stdout.log 
      // STDERR goes to stderr.log
      // Check stdout.log and stderr.log
      File stdout = new File(Atom.stdoutLog);
      if (stdout.exists()) {
	echoText(Atom.stdoutLog);
	stdout.delete();
      }
      File stderr = new File(Atom.stderrLog);
      if (stderr.exists()) {
	echoText(Atom.stderrLog, Atom.systemStatusTextArea);	
	stderr.delete();
      }
    }

    catch (Exception e) {
      String err = e.toString();
      System.err.println(err);
      System.err.println("Error occurred in FileIO.stopScl()");
      return;
    } 

  } // stopScl


  // *********************************************************************************
  // public void cleanUp() 
  //
  // Method Description:  This method will remove all of the temp files created by this
  // system.
  //
  // Method Specification:  This method will remove all of the temp files created by this
  // system.  First it checks to see if the temp file exists, and if it does it will
  // remove it.
  //   
  // This method will be accessed everytime system shutdown occurs.
  //
  // Pre-Condition:  True.
  //
  // Post-Condition: True.
  // 
  // *********************************************************************************

  public void cleanUp() {
  
    File file = new File(Atom.sclLevelTemp);
    if (file.exists()) {
      file.delete();
    }

    file = new File(Atom.sclTempFile);
    if (file.exists()) {
      file.delete();
    }  

    file = new File(Atom.auditTempFile);
    if (file.exists()) {
      file.delete();
    }  
 
  } // cleanUp


  // -----------------------------------------------------------------------------


  // *********************************************************************************
  // public List fillList (List list)
  //
  // Method Description:  This method retrieves each record from the scl_types.dat file
  // and fills a List with the Scl types name.
  //
  // Method Specification:  This method requires a source file, and a List to fill.
  // It will then proceed to fill the list with the names of the Scl types found in the
  // scl_types.dat file.  It will then return this list.
  //   
  // This method will be accessed several times.  Every time a tab/feature that has a
  // Scl list is activated, this method will be accessed.
  //
  // Pre-Condition:  The source file should exist. The List needs to be instantiated.
  //
  // Post-Condition: The text is added to the List and returned to the calling parent.
  // 
  // *********************************************************************************

  public List fillList (List list) {
    String token = "";
   
    File stuff = new File(Atom.sclTypesFile);

   // Check for file existence
   // If it does not exist inform the user
   if (!(stuff.exists())) {
     System.out.println(Atom.sclTypesFile);
     System.out.println("File Does Not Exist");
     return list;
   }
   try {
     BufferedReader sourceFile = new BufferedReader(new FileReader(Atom.sclTypesFile));

     // 
     // While the file is not EOF
     //    * Get a line of the source file
     //    * Parse the input line using the carot as a token delimiter
     //
     do {
       String str = sourceFile.readLine();   
       if (str == null) break;
       StringTokenizer inputString = new StringTokenizer(str, "^");        
       token = inputString.nextToken();
       list.addItem(token);    	   
     } while (true);

   }  // try 

   catch(Exception e) {
     String err = e.toString();
     System.out.println(err);
     System.out.println("Testing: just hit exception in FileIO.fillList(List)");
   } // catch

  return list;

 }   // fillList


  // *********************************************************************************
  // public List fillList (List list, String srcFile, String delimeter)
  //
  // Method Description:  This method retrieves each record from the scl_types.dat file
  // and fills a List with the Scl types name.
  //
  // Method Specification:  This method requires a source file, and a List to fill.
  // It will then proceed to fill the list with the names of the Scl types found in the
  // scl_types.dat file.  It will then return this list.
  //   
  // This method will be accessed several times.  Every time a tab/feature that has a
  // Scl list is activated, this method will be accessed.
  //
  // Pre-Condition:  The source file should exist. The List needs to be instantiated.
  //
  // Post-Condition: The text is added to the List and returned to the calling parent.
  // 
  // *********************************************************************************

  public List fillList (List list, String srcFile, String delimeter) {
    String token = "";

    // Check for file existence
    // If it does not exist inform the user
    File fileCheck = new File(srcFile);
    if(!(fileCheck.exists())) {
      return list;
    }

   try {
     BufferedReader sourceFile = new BufferedReader(new FileReader(srcFile));

     // 
     // While the file is not EOF
     //    * Get a line of the source file
     //    * Parse the input line using whitespace as a token delimiter
     //
     do {
       String completeToken = "";
       String str = sourceFile.readLine();
       if (str == null) break;
        StringTokenizer inputString = new StringTokenizer(str, " ");
        
          token = inputString.nextToken();
	 
	  // Make sure to get the full token or full name
	  if (token.equals(delimeter)) {                 
	    while (inputString.hasMoreTokens()) {
	      token = inputString.nextToken();	      
	      completeToken = completeToken+" "+token;	 
	    }
	    completeToken = completeToken.trim();
	    list.addItem(completeToken);    	    
	  }
     } while (true);
   }  // try 

   catch(Exception e) {
     String err = e.toString();
     System.out.println(err);
     System.out.println("Error occured in fillList(List, String, String)");
   } // catch

   return list;

  }   // fillList




  // --------------  Method Modification 4/8/98 Dean -----------

  public Choice fillChoice (Choice choice, String srcFile, String delimeter) {
    String token = "";

    // Check for file existence
    // If it does not exist inform the user
    File fileCheck = new File(srcFile);
    if(!(fileCheck.exists())) {
      return choice;
    }

   try {
     BufferedReader sourceFile = new BufferedReader(new FileReader(srcFile));

     // 
     // While the file is not EOF
     //    * Get a line of the source file
     //    * Parse the input line using whitespace as a token delimiter
     //
     do {
       String completeToken = "";
       String str = sourceFile.readLine();
       if (str == null) break;
        StringTokenizer inputString = new StringTokenizer(str, " ");
        
	token = inputString.nextToken();
	 
	// Make sure to get the full token or full name
	if (token.equals(delimeter)) {                 
	  while (inputString.hasMoreTokens()) {
	    token = inputString.nextToken();	      
	    completeToken = completeToken+" "+token;	 
	  }
	  completeToken = completeToken.trim();
	  choice.add(completeToken);    	    
	}
     } while (true);
   }  // try 

   catch(Exception e) {
     String err = e.toString();
     System.out.println(err);
     System.out.println("Error occured in fillChoice(Choice, String, String)");
   } // catch

   return choice;

  }   // fillChoice

} // FileIO






























