/*
 Copyright (C) 1997-1998 The HummingBird Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

// ***************************************************************************
// SclInterface Class
// 
//
// Class Description:  
// ------------------
// Purpose:  This class defines the layout and functionality of the Scl Interface.
// This interface is a seperate window from the main interface.  This class instantiates
// and lays out the GUI components for the Scl interface.  It implements its own 
// event handling and is capable of interacting with the whole system.  If this
// window is destroyed, only it will be destroyed.  This interface allows the
// user to add, remove, configure, assign, and un-assign security classification levels.
//
// System Relationship:  Intercommunication with the whole system is minimal.  The
// majority of system relationships involves updating List or Choice components 
// located on other interfaces.  For example, updating the Security Levels List (SCL Types) 
// on the Main Interface or the Atom.class.  File input/output is performed.
//
// Requirements:  This class requires the Java Developemnt Kit (JDK) 1.0.
// Aside from this there are no special algorithms, or constraints.  
//
// Pre-Conditions:  Atom.class.
//
// Post Conditions: True.
//
// Error Handling:  Addressed with the try and catch clause.
//
//
// Class Specification:
// --------------------
// This class must layout the interface components for the Scl Interface onto the 
// screen.  The Scl Interface appears on the console when the "SCL" button on the
// main interface is activated by the user.  This class will provide these options:
// add, remove, configure, assign, and un-assign.  These options will be layed out
// in a tabular format for the user to choose from.  This class must respond to user
// generated events.  The following is a list of requirements used by this class:
//
//    *  Instantiate java.awt components used by the GUI
//    *  Layout the java.awt components
//    *  Provide event handling for the java.awt components
//
//
// Constructor
// -----------
//      public SclInterface()
//
//
// Methods
// -------
//      public void layoutAddPanel()
//      public void layoutRemovePanel()
//      public void layoutConfigurePanel()
//      public void layoutAssignPanel()
//      public void layoutUnassignPanel()
//      public void constrain(Container container, Component component,
//                      int grid_x, int grid_y, int grid_width, int grid_height,
//                      int fill, int anchor, double weight_x, double weight_y,   
//                      int top, int left, int bottom, int right)
//      public void constrain(Container container, Component component,
//                      int grid_x, int grid_y, int grid_width, int grid_height) 
//      public void constrain(Container container, Component component,
//                      int grid_x, int grid_y, int grid_width, int grid_height,
//                      int top, int left, int bottom, int right)
//
// Event Handling 
// --------------
//      public boolean action(Event evt, Object arg)
//      public boolean handleEvent(Event evt)
//
//
// External Classes Used
// ---------------------
//      FileIO
//      Box
//
// *****************************************************************************

import java.io.*;
import java.awt.*;
import java.awt.event.*;

public class SclInterface extends Constrain implements ActionListener, ItemListener {

  // *******************************************
  // Interface Component Declarations
  // *******************************************
  Panel   tabs;                 // Sub-container for the buttons
  Panel   cards;                // Sub-container for the button features
  Panel   addPanel;             // Sub-container for the Add feature components
  Panel   removePanel;          // Sub-container for the Remove feature components
  Panel   configurePanel;       // Sub-container for the Configure feature components
  Panel   assignPanel;          // Sub-container for the Start feature components
  Panel   unassignPanel;        // Sub-container for the Stop feature components

  Button  addButton;            // Add button
  Button  removeButton;         // Remove button
  Button  configureButton;      // Configure button
  Button  assignButton;         // Assign button
  Button  unassignButton;       // Un-Assign button

  Button addSubmitButton;        // Add Submit Button
  Button removeSubmitButton;     // Remove Submit Button
  Button configureSubmitButton;  // Configure Submit Button
  Button assignSubmitButton;     // Assign Submit Button
  Button unassignSubmitButton;   // Un-Assign Submit Button

  TextField  addNameTextField;             // Add TextField for name
  TextField  assignParametersTextField;    // Assign TextField for parameters
  TextField  assignDirectoryTextField;     // Assign TextField for directory
  TextField  unassignParametersTextField;  // Un-Assign TextField for parameters

  TextArea   addDescriptionTextArea;       // Add TextArea for description
  TextArea   configureTextArea;            // Configure TextArea current configuration
  TextArea   configureTextAreaEdit;        // Configure TextArea for new configuration

  List  removeSclList;                     // Scl List for the Remove feature
  List  configureSclList;                  // Scl List for the Configure feature
  public static List  assignToolList;      // Tool List for the Assign feature
  public static List  unassignToolList;    // Tool List for the Un-Assign feature
  List  assignSclList;                     // Scl List for the Assign feature
  List  unassignSclList;                   // Scl List for the Un-Assign feature

  Choice    configureChoice;               // Choice menu for the Configure feature

  CardLayout singleCard   = new CardLayout();     // Layout Manager for the feature / cards
  GridBagLayout  gridbag  = new GridBagLayout();  // Layout Manager for the components
 
 
  // *******************************************
  // Data Declarations
  // *******************************************
  private int[]    listSelection = new int[0];             // List selection elements (integer indexes)
  private String[] alphaListSelection = new String[0];     // List selection elements (alpha name)
  private String[] secondaryAlphaListSelection = new String[0];  // List selection elements (alpha name)
  private String   fieldData;                                    // Variabe for component information
  private String   selectedItem;

  // *******************************************
  // Object instantiation
  // *******************************************
  FileIO sclIO      = new FileIO();                      

 
  // *****************************************************************************
  // public SclInterface()
  //
  // Constructor Description:  The constructor for the SclInterface  class.  Initializes
  // new variables and components, and lays out the components on the scl interface.
  //
  // Constructor Specification:  The constructor lays out the components for 
  // each area on the scl interface.  It uses the GridBagLayout Manager.  In addition
  // it defines the variables from the Data Declarations.  The following is a listing
  // of the interface layout and its corresponding components:
  // 
  //   Position  Description             Components
  //   --------  -----------             ----------
  //    * Frame  Scl Interface           GridBagLayout Manager
  //    * North  Tabs/Features Panel     GridLayout Manager, addButton, removeButton, 
  //                                     configureButton, assignButton, unassignButton
  //    * Center Cards                   CardLayout Manager
  //             Add                     addPanel, addNameTextField, addDescriptionTextArea 
  //             Remove                  removePanel, removeSclList, panel1
  //             Configure               configurePanel, configureSclList, configureTextArea,
  //                                     configureTextAreaEdit, configureChoice, panel1, panel2
  //             Assign                  assignPanel, assignSclList, assignToolList, 
  //                                     assignParametersTextField, assignDirectoryTextField,
  //                                     panel1, panel2, panel3
  //                                     startCheckBox, panel1, panel2
  //             Un-Assign               unassignPanel, unassignSclList, unassignToolList, 
  //                                     unassignParametersTextField, panel1, panel2, panel3            
  //
  // The constructor will be accessed each time the SclInterface, is instantiated.
  // 
  // Pre-Conditions:  True.
  //
  // Post-Conditions: True.
  //
  // *****************************************************************************

  public SclInterface () {
    setTitle("SCL Options");

    addWindowListener(new WinAdapter());
    
    // *** Initialize Buttons ****
    addButton        = new Button("Add");
    removeButton     = new Button("Remove");
    configureButton  = new Button("Configure");
    assignButton     = new Button("Assign");
    unassignButton   = new Button("UnAssign");

    // *** BUTTON LISTENERS ***
    addButton.addActionListener(this);
    removeButton.addActionListener(this);
    configureButton.addActionListener(this);
    assignButton.addActionListener(this);
    unassignButton.addActionListener(this);

    // *** Set Font and Color of Buttons ****
    addButton.setFont(new Font("Dialog", Font.BOLD, 14));
    removeButton.setFont(new Font("Dialog", Font.BOLD, 14));
    configureButton.setFont(new Font("Dialog", Font.BOLD, 14));
    assignButton.setFont(new Font("Dialog", Font.BOLD, 14));
    unassignButton.setFont(new Font("Dialog", Font.BOLD, 14));

    // *** Tabs ****
    tabs = new Panel();
    tabs.setLayout(new GridLayout(0,6));
     tabs.add(addButton);
     tabs.add(removeButton);   
     tabs.add(configureButton);
     tabs.add(assignButton);
     tabs.add(unassignButton);
    add("North", tabs);

    // *** Layout Card Method ****
    layoutAddPanel();

    // *** Cards ****
    cards = new Panel();
    cards.setLayout(singleCard);
    cards.add("Add",       addPanel);
    add("Center", cards);

  }  // Constructor


  // ***************************************************************************** 
  // public boolean action(Event evt, Object arg)
  //
  // Method Description:  This method handles all of the events (ACTION_EVENT) 
  // generated by the components in the scl interface.
  //
  // Method Specification:  If an ACTION_EVENT occurs this method will handle it.
  // Events are generated by the user activating a component.  Such as, clicking a
  // button, or selecting a list item.  The following lists the events and reactions
  // for each ACTION_EVENT:
  //
  //        Label     Component/Event       Reaction
  //        -----     ---------------       --------
  //      * Add       addSubmitButton       Add a security classification level to the
  //                                        scl_types.dat
  //      * Remove    removeSubmitButton    Remove a security classification level from 
  //                                        the scl_types.dat
  //      * Configure configureSubmitButton Modify the user selected field of the security
  //                                        classification level
  //      * Assign    assignSubmitButton    Assign an audit tool to a security classification
  //                                        level
  //      * Un-Assign unassignSubmitButton  Un-Assign an audit tool from a security 
  //                                        classification level
  //
  // This method will be accessed many times.
  //
  // Pre-Condition:  Component objects have been instantiated.
  //
  // Post-Condition: True. Events handle properly.
  //
  // *****************************************************************************

  public void actionPerformed(ActionEvent evt) {
    Object source = evt.getSource();
    
    String nameInfo = "";
    String descriptionInfo = "";
    String info = "";
    String dirInfo = "";
    int tempIndex;


    // -------------------
    // Add Submit Button
    // -------------------
    if (source == addSubmitButton) {
             
      // Collect information from the TextFields
      // Collection Format:  <name> <description>
      nameInfo        = addNameTextField.getText();
      descriptionInfo = addDescriptionTextArea.getText();

      // Validity check
      if (nameInfo.equals("")) {
	Atom.systemStatusTextArea.append("The NAME field is required to Add a Security Classification Level \n");
      }
      else {   
	  
	// Put the Scl information into the scl_types.dat file       
	sclIO.AtplAppendText(Atom.sclTypesFile, ToolInterface.NAME, nameInfo);
	sclIO.AtplAppendText(Atom.sclTypesFile, ToolInterface.DESCRIPTION, descriptionInfo);
	sclIO.AtplAppendText(Atom.sclTypesFile, ToolInterface.END, ToolInterface.ENDSYMBOL);

	// Update security levels List in the Atom class
	Atom.sclTypesList.add(nameInfo);
	
	// The ToolInterface class may not have been instantiated yet, so
	// check and handle these lists if possible.
	try {
	  ToolInterface.addChoice.add(nameInfo);
	  ToolInterface.startChoice.add(nameInfo);
	}
	catch (NullPointerException e) {
	  String err = e.toString();
	}

	// Clear the text fields
	addNameTextField.setText("");
	addDescriptionTextArea.setText("");
      }
    } // add
    

    // -----------
    // Remove 
    // -----------
    if (source == removeSubmitButton) {

      // Validity check
      if (selectedItem == null) {
	Atom.systemStatusTextArea.append("Please select an item to Remove. \n");
      }
	else {

	  // The user has selected a Scl to remove so do the following
	  //     * Remove the item from the sclTypesFile
	  //     * Remove the item from all the Scl types lists
	  //
		  
	  removeSclList.remove(selectedItem);
	  Atom.sclTypesList.remove(selectedItem);
	    
	  // Remove scl_type file
	  String file = Atom.sclLevelDir+"scl_"+selectedItem;
	    
	  // When the user deletes an item if there is a corresponding Scl Level Assignment
	  // file, then remove it.
	  File removeFile = new File(file);
	  if (removeFile.exists()) {
	    removeFile.delete();
	  }

	  // Update scl_types.dat file
	  sclIO.AtplRemoveRecord(Atom.sclTypesFile, Atom.sclTempFile, selectedItem);
	} // else
    } // remove
    
    // -----------
    // Configure
    // -----------
    if (source == configureSubmitButton) {      
      String choiceItem = "";

      // Collect information from the Choice and TextArea
      choiceItem = configureChoice.getSelectedItem();
      info = configureTextAreaEdit.getText();             // textarea information <name> or <description>

      // Validity check
      if (alphaListSelection.length > 0) {         
	nameInfo = alphaListSelection[0];           
      }
      else {
	Atom.systemStatusTextArea.append("Please select an item from the list to configure.\n");
      }

      // *** remove when scl_types.dat is converted to atpl format
      info = info.replace('\n',' ');
      info = info.trim();

      if (choiceItem.equals("NAME")) {

	// Validity check on the name
	if (info.equals("")) {
	  Atom.systemStatusTextArea.append("Please enter a new name.\n");
	}
	else {
        
	  //
	  // The user selected "Name" from the Choice menu, so
	  //    * Get information about the Scl from the scl_types.dat file
	  //    * Replace the record with the appropriate information, in the scl_types.dat file
	  //    * Clear the TextAreas 
	  //    * Update the Scl Lists
	  // 
	  fieldData = sclIO.getText(Atom.sclTypesFile, alphaListSelection[0]);
	  sclIO.replaceText(Atom.sclTypesFile, Atom.sclTempFile, nameInfo, info, fieldData); 
	  configureTextAreaEdit.setText("");
	  configureTextArea.setText("");

	  // Update scl lists
	  tempIndex = configureSclList.getSelectedIndex();
	  Atom.sclTypesList.remove(tempIndex);
	  Atom.sclTypesList.add(info);
	  configureSclList.remove(tempIndex);
	  configureSclList.add(info);
	}
      }

      if (choiceItem.equals("DESCRIPTION")) {
	
	// Validity check on the description, 
	// If no user description then default to "No description"
	if (info.equals("")) {
	  info = "No description";
	}

	//
	// The user selected "Description" from the Choice menu, so
	//    * Replace the record with the appropriate information, in the scl_types.dat file
	//    * Clear the TextAreas
	sclIO.replaceText(Atom.sclTypesFile, Atom.sclTempFile, nameInfo, nameInfo, info);
	configureTextArea.setText("");
	configureTextAreaEdit.setText("");
      }
              
      // Display the new configuration in the TextAreas
      fieldData = sclIO.getText(Atom.sclTypesFile, alphaListSelection[0]);
      configureTextArea.setText(fieldData);
    } // configure

    
    // -----------
    // Assign
    // -----------
    if (source  == assignSubmitButton) {             
      info = assignParametersTextField.getText();
      dirInfo = assignDirectoryTextField.getText();

      // Validity check
      if ((info.equals("")) || (dirInfo.equals(""))) {
	Atom.systemStatusTextArea.append("Please enter information in Directory and Parameters Text Fields. \n");
      }
		
      // 
      // Each list selection variable corresponds to:
      //         alphaListSelection            =>  audit_tool     
      //         secondaryAlphaListSelection   =>  scl_type
      //
      if ((alphaListSelection.length > 0) && (secondaryAlphaListSelection.length > 0)) { 
	sclIO.appendSclAssignment(alphaListSelection[0], secondaryAlphaListSelection[0],
				  info, dirInfo);                                        
	assignParametersTextField.setText("");
	assignDirectoryTextField.setText("");
      }
      else {
	Atom.systemStatusTextArea.append("Please select items from both lists to Assign.\n");
      }
    } // assign


    // -----------
    // Un-Assign
    // -----------
    if (source == unassignSubmitButton) {      

      // Each list selection variable corresponds to:
      //       alphaListSelection            =>  audit_tool         
      //       secondaryAlphaListSelection   =>  scl_type
      //
      if ((alphaListSelection.length > 0) && (secondaryAlphaListSelection.length > 0)) { 
	sclIO.removeSclAssignment(alphaListSelection[0], secondaryAlphaListSelection[0]);
	unassignParametersTextField.setText("");
      }
      else {
	Atom.systemStatusTextArea.append("Please select an item from the list to Un-Assign.\n");
      }
    } // un-assign


    // ----------
    // Cards
    // ----------
    

    if (source instanceof Component &&
	((Component)evt.getSource()).getParent().equals(tabs)) {

      String tabType = evt.getActionCommand();
       
      //
      // If the user select a tab/feature display the apporpriate card, moreover
      // display the card/features interface
      //
      if (tabType.equals("Remove")) {
	layoutRemovePanel();
	selectedItem=null;
	cards.add("Remove",    removePanel);
      }
    
      if (tabType.equals("Configure")) {
	layoutConfigurePanel();
	cards.add("Configure",    configurePanel);
	alphaListSelection = new String[0];
	secondaryAlphaListSelection = new String[0];
      }       

      if (tabType.equals("Assign")) {
	layoutAssignPanel();
	cards.add("Assign",    assignPanel);
	alphaListSelection = new String[0];
	secondaryAlphaListSelection = new String[0];
      }       
       
      if (tabType.equals("UnAssign")) {
	layoutUnassignPanel();
	cards.add("UnAssign",    unassignPanel);
	alphaListSelection = new String[0];
	secondaryAlphaListSelection = new String[0];
      }       
      
      singleCard.show(cards, tabType);
    }
  } // action



class WinAdapter extends WindowAdapter {
  public void windowClosing(WindowEvent event) {
    dispose();
  }
}


  // *********************************************************************************
  // public boolean handleEvent(Event evt)
  //
  // Method Description:  This method handles all of the events that are not of type
  // ACTION_EVENT, generated by the components in the scl interface.  The events will
  // include LIST_SELECT, LIST_DESELECT, and WINDOW_DESTROY.
  //
  // Method Specification:  If one of the preceeding events occurs this method will 
  // handle it.  Events are generated by the user activating a component.  Such as, 
  // selecting or deselecting list elements, or destroying the current window/frame.
  // The following lists the events and reactions for each event:
  //
  //        Action Type     Component/Event        Reaction
  //        -----------     ---------------        --------
  //      * LIST_SELECT     removeSclList          listSelection = selected indexes
  //                        configureSclList       alphaListSelection = selected items
  //                        assignToolList         alphaListSelection = selected items
  //                        assignSclList          secondaryAlphaListSelection = selected items
  //                        unassignToolList       alphaListSelection = selected items
  //                        unassignSclList        secondaryAlphaListSelection = selected items
  //      * LIST_DESELECT   removeSclList          listSelection = selected indexes
  //                        configureSclList       alphaListSelection = selected items
  //                        assignToolList         secondaryAlphaListSelection = selected items
  //                        unassignToolList       alphaListSelection = selected items
  //                        unassignSclList        secondaryAlphaListSelection = selected items
  //      * WINDOW_DESTROY  this                   exit
  //
  // This method will be accessed many times.  Only the WINDOW_DESTROY will be 
  // accessed once.
  //
  // Pre-Condition:  Component objects have been instantiated.
  //
  // Post-Condition: True.  Lists have proper component names or indexes.
  // 
  // *********************************************************************************

  public void itemStateChanged(ItemEvent evt) {

    //
    //    if (evt.id == Event.WINDOW_DESTROY) {
    //      this.dispose();
    //    }
    
    String fieldData = "";
    String dirData = "";

    switch(evt.getStateChange()) {
    case ItemEvent.SELECTED:

      // Remove
      if (evt.getSource().equals(removeSclList)) {
	selectedItem = removeSclList.getSelectedItem();
      }
      
      // Configure
      if (evt.getSource().equals(configureSclList)) {
	alphaListSelection = configureSclList.getSelectedItems();
	configureTextArea.setText("");
	fieldData = sclIO.getText(Atom.sclTypesFile, alphaListSelection[0]);
	configureTextArea.append(fieldData);
      }

      // Assign
      if (evt.getSource().equals(assignToolList)) {
	alphaListSelection = assignToolList.getSelectedItems();
	assignParametersTextField.setText("");
	assignDirectoryTextField.setText("");
	fieldData = sclIO.AtplGetField(Atom.auditToolsFile, alphaListSelection[0], 
				       ToolInterface.DEFAULT);
	dirData   = sclIO.AtplGetField(Atom.auditToolsFile, alphaListSelection[0], 
				       ToolInterface.DIRECTORY);	
	assignParametersTextField.setText(fieldData);
	assignDirectoryTextField.setText(dirData);
      }

      if (evt.getSource().equals(assignSclList)) {
	secondaryAlphaListSelection = assignSclList.getSelectedItems();
      }

      // Un-Assign
      if (evt.getSource().equals(unassignToolList)) {
	alphaListSelection = unassignToolList.getSelectedItems();
	unassignParametersTextField.setText("");
	fieldData = sclIO.AtplGetField(Atom.auditToolsFile, alphaListSelection[0], 
				       ToolInterface.DEFAULT);
	unassignParametersTextField.setText(fieldData);
      }
      
      if (evt.getSource().equals(unassignSclList)) {
	secondaryAlphaListSelection = unassignSclList.getSelectedItems();
      }     

      break;
      
    case ItemEvent.DESELECTED:
      
      // Configure
      if (evt.getSource().equals(configureSclList)) {
	alphaListSelection = configureSclList.getSelectedItems();
      }
      
      // Assign
      if (evt.getSource().equals(assignToolList)) {
	alphaListSelection = assignToolList.getSelectedItems();
      }

      if (evt.getSource().equals(assignSclList)) {
	secondaryAlphaListSelection = assignSclList.getSelectedItems();
      }

      // Un-Assign
      if (evt.getSource().equals(unassignToolList)) {
	alphaListSelection = unassignToolList.getSelectedItems();
      }

      if (evt.getSource().equals(unassignSclList)) {
	secondaryAlphaListSelection = unassignSclList.getSelectedItems();
      }

    break;
    }

  } // handleEvent


  // *********************************************************************************
  // public void layoutAddPanel ()
  //
  // Method Description:  This method lays out all of the components for the Add tab,
  // located on the scl interface.
  //
  // Method Specification:  When the Add tab is activated by the user, the corresponding
  // Add feature interface needs to be displayed.  This method instantiates all of the 
  // components and positions them using the GridBagLayout manager.
  //   
  // This method will be accessed several times.  Every time the Add tab is activated this
  // method will be called.
  //
  // Pre-Condition:  Add components have been declared.
  //
  // Post-Condition: True.  The components have been instantiated and layed out.
  // 
  // *********************************************************************************

  public void layoutAddPanel () {
    addPanel                 = new Panel();
    addNameTextField         = new TextField(40);
    addDescriptionTextArea   = new TextArea(6,60);

     addPanel = new Panel();    
     addPanel.setLayout(gridbag);
     addSubmitButton = new Button("Submit");
     addSubmitButton.setFont(new Font("Dialog", Font.BOLD, 12));
     addSubmitButton.setForeground(Color.blue);

     // ADD LISTENERS
     addSubmitButton.addActionListener(this);

     // The constrain method places the components onto the interface using the
     // GridBagLayout manager. 
     constrain(addPanel, new Label("Name:"), 0,0,1,1, 20,20,0,0);
     constrain(addPanel, addNameTextField, 0,1,1,1, 0,20,0,0);     
     constrain(addPanel, new Label("Description:"), 0,2,1,1, 10,20,0,0);
     constrain(addPanel, addDescriptionTextArea, 0,3,1,1, 0,20,0,0);
     constrain(addPanel, addSubmitButton, 0,3,1,1, GridBagConstraints.NONE,
	       GridBagConstraints.CENTER, 1.0, 1.0, 20, 0, 5, 5);   

  } // layoutAddPanel


  // *********************************************************************************
  // public void layoutRemovePanel ()
  //
  // Method Description:  This method lays out all of the components for the Remove tab,
  // located on the scl interface.
  //
  // Method Specification:  When the Remove tab is activated by the user, the corresponding
  // Remove feature interface needs to be displayed.  This method instantiates all of the 
  // components and positions them using the GridBagLayout manager.
  //   
  // This method will be accessed several times.  Every time the Remove tab is activated 
  // this method will be called.
  //
  // Pre-Condition:  Remove components have been declared in the Interface Components 
  // Declarations.
  //
  // Post-Condition: True.  The components have been instantiated and layed out.
  // 
  // *********************************************************************************

  public void layoutRemovePanel() {
    removePanel         = new Panel();            
    removeSclList       = new List(15, false);       
    Panel panel1        = new Panel();

     panel1.setLayout(gridbag);
     removePanel.setLayout(gridbag);
     removeSubmitButton = new Button("Submit");
     removeSubmitButton.setFont(new Font("Dialog", Font.BOLD, 12));
     removeSubmitButton.setForeground(Color.blue);   

     // ADD LISTENERS
     removeSubmitButton.addActionListener(this);
     removeSclList.addItemListener(this);

     // Fill the removeSclList with the names of the Scl types 
     removeSclList = sclIO.fillList(removeSclList, Atom.sclTypesFile, ToolInterface.NAME);

     // The constrain method places the components onto the interface using the
     // GridBagLayout manager.      
     constrain(panel1, removeSclList, 0,0,1,1);
     Box panelBox = new Box(panel1, "Security Levels");
     panelBox.etchedOut();
     
     constrain(removePanel, panelBox, 0,0,1,1);
     constrain(removePanel, removeSubmitButton, 0,1,1,1, 20,40,0,0);

  } // layoutRemovePanel


  // *********************************************************************************
  // public void layoutConfigurePanel ()
  //
  // Method Description:  This method lays out all of the components for the Configure tab,
  // located on the tool interface.
  //
  // Method Specification:  When the Configure tab is activated by the user, the corresponding
  // Configure feature interface needs to be displayed.  This method instantiates all of the 
  // components and positions them using the GridBagLayout manager.
  //   
  // This method will be accessed several times.  Every time the Configure tab is activated 
  // this method will be called.
  //
  // Pre-Condition:  Configure components have been declared in the Interface Components 
  // Declarations.
  //
  // Post-Condition: True.  The components have been instantiated and layed out.
  // 
  // *********************************************************************************

  public void layoutConfigurePanel() {
    configurePanel              = new Panel();
    configureSclList            = new List(22, false);
    configureTextArea           = new TextArea(8,50);
    configureTextAreaEdit       = new TextArea(8,50);
    configureChoice             = new Choice();

    Panel     panel1            = new Panel();
    Panel     panel2            = new Panel(); 
       
    configurePanel.setLayout(gridbag);
    panel1.setLayout(gridbag);
    panel2.setLayout(gridbag);
    configureSubmitButton = new Button("Submit");
    configureSubmitButton.setFont(new Font("Dialog", Font.BOLD, 12));
    configureSubmitButton.setForeground(Color.blue);

    // ADD LISTENERS
    configureSubmitButton.addActionListener(this);
    configureSclList.addItemListener(this);

    // Fill the configureSclList with the names of the Scl types 
    configureSclList = sclIO.fillList(configureSclList, Atom.sclTypesFile, ToolInterface.NAME);

    // This is where the Choice menu is filled with choices
    configureChoice.add("NAME");
    configureChoice.add("DESCRIPTION");

    // The constrain method places the components onto the interface using the
    // GridBagLayout manager.      
    constrain(panel1, configureSclList, 0,0,1,1);
    Box panelBox = new Box(panel1, "Security Levels");
    panelBox.etchedOut();

    constrain(panel2, configureChoice, 0,0,1,1, 10,0,0,0);
    constrain(panel2, new Label("Current Configuration"), 0,1,1,1, 20,0,0,0);
    constrain(panel2, configureTextArea, 0,2,1,1);
    constrain(panel2, new Label("New Configuration"), 0,3,1,1, 10,0,0,0);
    constrain(panel2, configureTextAreaEdit, 0,4,1,1); 
    constrain(panel2, configureSubmitButton, 0,5,1,1, GridBagConstraints.NONE,
	      GridBagConstraints.CENTER, 1.0, 1.0, 10, 0, 5, 5);

    constrain(configurePanel, panelBox, 0,0,1,1, GridBagConstraints.HORIZONTAL,
	      GridBagConstraints.NORTH, 1.0, 1.0, 1, 10, 5, 10);
    constrain(configurePanel, panel2, 1,0,1,1, GridBagConstraints.HORIZONTAL,
	      GridBagConstraints.NORTH, 1.0, 1.0, 1, 10, 5, 10);
    
  } // layoutConfigurePanel


  // *********************************************************************************
  // public void layouAssignPanel ()
  //
  // Method Description:  This method lays out all of the components for the Assign tab,
  // located on the tool interface.
  //
  // Method Specification:  When the Assign tab is activated by the user, the corresponding
  // Assign feature interface needs to be displayed.  This method instantiates all of the 
  // components and positions them using the GridBagLayout manager.
  //   
  // This method will be accessed several times.  Every time the Assign tab is activated 
  // this method will be called.
  //
  // Pre-Condition:  Assign components have been declared in the Interface Components 
  // Declarations.
  //
  // Post-Condition: True.  The components have been instantiated and layed out.
  // 
  // *********************************************************************************

  public void layoutAssignPanel() {
    assignPanel                     = new Panel();
    assignSclList                   = new List(15, false);
    assignToolList                  = new List(15, false);
    assignParametersTextField       = new TextField(60);
    assignDirectoryTextField        = new TextField(60);

    Panel      panel1               = new Panel();
    Panel      panel2               = new Panel();
    Panel      panel3               = new Panel();

     assignPanel.setLayout(gridbag);
     panel1.setLayout(gridbag);
     panel2.setLayout(gridbag);
     panel3.setLayout(gridbag);

     assignSubmitButton = new Button("Submit");
     assignSubmitButton.setFont(new Font("Dialog", Font.BOLD, 12));
     assignSubmitButton.setForeground(Color.blue);

     // ADD LISTENERS
     assignSubmitButton.addActionListener(this);
     assignToolList.addItemListener(this);
     assignSclList.addItemListener(this);

     // Fill the assignToolList with the names of the registered audit tools 
     // Fill the assignSclList with the names of the Scl types 
     assignToolList = sclIO.fillList(assignToolList, Atom.auditToolsFile, ToolInterface.NAME);
     assignSclList  = sclIO.fillList(assignSclList,  Atom.sclTypesFile, ToolInterface.NAME);
     

     // The constrain method places the components onto the interface using the
     // GridBagLayout manager.      
     constrain(panel1, assignToolList, 0,0,1,1);
     Box panelBox = new Box(panel1, "Audit Tools");
     panelBox.etchedOut();

     constrain(panel2, new Label("Directory:"), 0,0,1,1);
     constrain(panel2, assignDirectoryTextField, 0,1,1,1);
     constrain(panel2, new Label("Parameters:"), 0,2,1,1);
     constrain(panel2, assignParametersTextField, 0,3,1,1);
     constrain(panel2, assignSubmitButton, 0,4,1,1, GridBagConstraints.NONE,
	       GridBagConstraints.CENTER, 1.0, 1.0, 10, 0, 5, 5);

     constrain(panel3, assignSclList, 0,0,1,1);
     Box panel3Box = new Box(panel3, "Security Levels");
     panel3Box.etchedOut();
 
     constrain(assignPanel, panelBox, 0,0,1,1, GridBagConstraints.NONE,
	       GridBagConstraints.NORTH, 1.0, 1.0, 10, 10, 0, 10);
     constrain(assignPanel, panel3Box, 1,0,1,1, GridBagConstraints.NONE,
	       GridBagConstraints.NORTH, 1.0, 1.0, 10, 10, 0, 10);
     constrain(assignPanel, panel2, 0,1,2,1, GridBagConstraints.HORIZONTAL,
	       GridBagConstraints.SOUTH, 1.0, 1.0, 10, 20, 0, 0);
         
  } // layoutAssignPanel


  // *********************************************************************************
  // public void layouUnassignPanel ()
  //
  // Method Description:  This method lays out all of the components for the Unassign tab,
  // located on the tool interface.
  //
  // Method Specification:  When the Unassign tab is activated by the user, the corresponding
  // Unassign feature interface needs to be displayed.  This method instantiates all of the 
  // components and positions them using the GridBagLayout manager.
  //   
  // This method will be accessed several times.  Every time the Unassign tab is activated 
  // this method will be called.
  //
  // Pre-Condition:  Unassign components have been declared in the Interface Components 
  // Declarations.
  //
  // Post-Condition: True.  The components have been instantiated and layed out.
  // 
  // *********************************************************************************

  public void layoutUnassignPanel() {
    unassignPanel                 = new Panel();             
    unassignSclList               = new List(15, false);
    unassignToolList              = new List(15, false);
    unassignParametersTextField   = new TextField(60);

    Panel panel1          = new Panel();
    Panel panel2          = new Panel();    
    Panel panel3          = new Panel();

     panel1.setLayout(gridbag);
     panel2.setLayout(gridbag);
     panel3.setLayout(gridbag);
     unassignPanel.setLayout(gridbag);

     unassignSubmitButton = new Button("Submit");
     unassignSubmitButton.setFont(new Font("Dialog", Font.BOLD, 12));
     unassignSubmitButton.setForeground(Color.blue);
     
     // ADD LISTENERS
     assignSubmitButton.addActionListener(this);
     assignToolList.addItemListener(this);
     assignSclList.addItemListener(this);

     // Fill the unassignToolList with the names of the registered audit tools 
     // Fill the unassignSclList with the names of the Scl types 
     unassignToolList = sclIO.fillList(unassignToolList, Atom.auditToolsFile, 
					ToolInterface.NAME);
     unassignSclList  = sclIO.fillList(unassignSclList,  Atom.sclTypesFile, 
				       ToolInterface.NAME);

     // The constrain method places the components onto the interface using the
     // GridBagLayout manager.           
     constrain(panel1, unassignToolList, 0,0,1,1);
     Box panelBox = new Box(panel1, "Audit Tools");
     panelBox.etchedOut();

     constrain(panel2, unassignSclList, 0,0,1,1);
     Box panel2Box = new Box(panel2, "Security Levels");
     panel2Box.etchedOut();

     constrain(panel3, new Label("Parameters:"), 0,0,1,1);
     constrain(panel3, unassignParametersTextField, 0,1,1,1);
     constrain(panel3, unassignSubmitButton, 0,2,1,1, GridBagConstraints.NONE,
	       GridBagConstraints.CENTER, 1.0, 1.0, 10, 0, 5, 5);
   
     constrain(unassignPanel, panelBox, 0,0,1,1, GridBagConstraints.NONE,
	       GridBagConstraints.NORTH, 1.0, 1.0, 1, 10, 5, 10);
     constrain(unassignPanel, panel2Box, 1,0,1,1, GridBagConstraints.NONE,
	       GridBagConstraints.NORTH, 1.0, 1.0, 1, 10, 5, 10);
     constrain(unassignPanel, panel3, 0,1,2,1, GridBagConstraints.HORIZONTAL,
	       GridBagConstraints.SOUTH, 1.0, 1.0, 0, 20, 0, 0);

  }  // layoutUnassignPanel

}













 



